@@ -1,20 +1,10 @@
 /*
- * Copyright (C) 2013 - Julien Desfossez <jdesfossez@efficios.com>
- *                      David Goulet <dgoulet@efficios.com>
- *               2015 - Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright (C) 2013 Julien Desfossez <jdesfossez@efficios.com>
+ * Copyright (C) 2013 David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2015 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License, version 2 only, as
- * published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -41,6 +31,29 @@ end:
 	return vsession;
 }
 
+int viewer_session_set_trace_chunk_copy(struct relay_viewer_session *vsession,
+		struct lttng_trace_chunk *relay_session_trace_chunk)
+{
+	int ret = 0;
+	struct lttng_trace_chunk *viewer_chunk;
+
+	assert(relay_session_trace_chunk);
+	lttng_trace_chunk_put(vsession->current_trace_chunk);
+	vsession->current_trace_chunk = NULL;
+
+	DBG("Copying relay session's current trace chunk to the viewer session");
+	viewer_chunk = lttng_trace_chunk_copy(relay_session_trace_chunk);
+	if (!viewer_chunk) {
+		ERR("Failed to create a viewer trace chunk from the relay session's current chunk");
+		ret = -1;
+		goto end;
+	}
+
+	vsession->current_trace_chunk = viewer_chunk;
+end:
+	return ret;
+}
+
 /* The existence of session must be guaranteed by the caller. */
 enum lttng_viewer_attach_return_code viewer_session_attach(
 		struct relay_viewer_session *vsession,
@@ -65,7 +78,7 @@ enum lttng_viewer_attach_return_code viewer_session_attach(
 		assert(!vsession->current_trace_chunk);
 		session->viewer_attached = true;
 
-		ret = viewer_session_set_trace_chunk(vsession,
+		ret = viewer_session_set_trace_chunk_copy(vsession,
 				session->current_trace_chunk);
 		if (ret) {
 			/*
@@ -205,25 +218,3 @@ end:
 	pthread_mutex_unlock(&session->lock);
 	return found;
 }
-
-int viewer_session_set_trace_chunk(struct relay_viewer_session *vsession,
-		struct lttng_trace_chunk *relay_session_trace_chunk)
-{
-	int ret = 0;
-	struct lttng_trace_chunk *viewer_chunk;
-
-	assert(relay_session_trace_chunk);
-	assert(!vsession->current_trace_chunk);
-
-	DBG("Copying relay session's current trace chunk to the viewer session");
-	viewer_chunk = lttng_trace_chunk_copy(relay_session_trace_chunk);
-	if (!viewer_chunk) {
-		ERR("Failed to create a viewer trace chunk from the relay session's current chunk");
-		ret = -1;
-		goto end;
-	}
-
-	vsession->current_trace_chunk = viewer_chunk;
-end:
-	return ret;
-}
