@@ -1,19 +1,9 @@
 /*
- * Copyright (C) 2011 - David Goulet <david.goulet@polymtl.ca>
- * Copyright (C) 2016 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ * Copyright (C) 2011 David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -55,7 +45,7 @@ static int add_kctx_all_channels(struct ltt_kernel_session *ksession,
 		kctx_copy = trace_kernel_copy_context(kctx);
 		if (!kctx_copy) {
 			PERROR("zmalloc ltt_kernel_context");
-			ret = -ENOMEM;
+			ret = -LTTNG_ERR_NOMEM;
 			goto error;
 		}
 
@@ -107,7 +97,8 @@ error:
  */
 static int add_uctx_to_channel(struct ltt_ust_session *usess,
 		enum lttng_domain_type domain,
-		struct ltt_ust_channel *uchan, struct lttng_event_context *ctx)
+		struct ltt_ust_channel *uchan,
+		const struct lttng_event_context *ctx)
 {
 	int ret;
 	struct ltt_ust_context *uctx = NULL;
@@ -119,7 +110,7 @@ static int add_uctx_to_channel(struct ltt_ust_session *usess,
 	/* Check if context is duplicate */
 	cds_list_for_each_entry(uctx, &uchan->ctx_list, list) {
 		if (trace_ust_match_context(uctx, ctx)) {
-			ret = -EEXIST;
+			ret = LTTNG_ERR_UST_CONTEXT_EXIST;
 			goto duplicate;
 		}
 	}
@@ -140,7 +131,7 @@ static int add_uctx_to_channel(struct ltt_ust_session *usess,
 		if (!agt) {
 			agt = agent_create(domain);
 			if (!agt) {
-				ret = LTTNG_ERR_NOMEM;
+				ret = -LTTNG_ERR_NOMEM;
 				goto error;
 			}
 			agent_add(agt, usess->agents);
@@ -165,22 +156,23 @@ static int add_uctx_to_channel(struct ltt_ust_session *usess,
 	/* Create ltt UST context */
 	uctx = trace_ust_create_context(ctx);
 	if (uctx == NULL) {
-		ret = -EINVAL;
+		ret = LTTNG_ERR_UST_CONTEXT_INVAL;
 		goto error;
 	}
 
+	/* Add ltt UST context node to ltt UST channel */
+	lttng_ht_add_ulong(uchan->ctx, &uctx->node);
+	cds_list_add_tail(&uctx->list, &uchan->ctx_list);
+
+	if (!usess->active) {
+		goto end;
+	}
+
 	ret = ust_app_add_ctx_channel_glb(usess, uchan, uctx);
 	if (ret < 0) {
 		goto error;
 	}
-
-	rcu_read_lock();
-
-	/* Add ltt UST context node to ltt UST channel */
-	lttng_ht_add_ulong(uchan->ctx, &uctx->node);
-	rcu_read_unlock();
-	cds_list_add_tail(&uctx->list, &uchan->ctx_list);
-
+end:
 	DBG("Context UST %d added to channel %s", uctx->ctx.ctx, uchan->name);
 
 	return 0;
@@ -195,7 +187,7 @@ duplicate:
  * Add kernel context to tracer.
  */
 int context_kernel_add(struct ltt_kernel_session *ksession,
-		struct lttng_event_context *ctx, char *channel_name)
+		const struct lttng_event_context *ctx, char *channel_name)
 {
 	int ret;
 	struct ltt_kernel_channel *kchan;
@@ -207,7 +199,7 @@ int context_kernel_add(struct ltt_kernel_session *ksession,
 
 	kctx = trace_kernel_create_context(NULL);
 	if (!kctx) {
-		ret = LTTNG_ERR_NOMEM;
+		ret = -LTTNG_ERR_NOMEM;
 		goto error;
 	}
 
@@ -259,6 +251,69 @@ int context_kernel_add(struct ltt_kernel_session *ksession,
 	case LTTNG_EVENT_CONTEXT_MIGRATABLE:
 		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_MIGRATABLE;
 		break;
+	case LTTNG_EVENT_CONTEXT_CALLSTACK_KERNEL:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_CALLSTACK_KERNEL;
+		break;
+	case LTTNG_EVENT_CONTEXT_CALLSTACK_USER:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_CALLSTACK_USER;
+		break;
+	case LTTNG_EVENT_CONTEXT_CGROUP_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_CGROUP_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_IPC_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_IPC_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_MNT_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_MNT_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_NET_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_NET_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_PID_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_PID_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_USER_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_USER_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_UTS_NS:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_UTS_NS;
+		break;
+	case LTTNG_EVENT_CONTEXT_UID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_UID;
+		break;
+	case LTTNG_EVENT_CONTEXT_EUID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_EUID;
+		break;
+	case LTTNG_EVENT_CONTEXT_SUID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_SUID;
+		break;
+	case LTTNG_EVENT_CONTEXT_GID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_GID;
+		break;
+	case LTTNG_EVENT_CONTEXT_EGID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_EGID;
+		break;
+	case LTTNG_EVENT_CONTEXT_SGID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_SGID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VUID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VUID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VEUID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VEUID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VSUID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VSUID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VGID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VGID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VEGID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VEGID;
+		break;
+	case LTTNG_EVENT_CONTEXT_VSGID:
+		kctx->ctx.ctx = LTTNG_KERNEL_CONTEXT_VSGID;
+		break;
 	default:
 		ret = LTTNG_ERR_KERN_CONTEXT_FAIL;
 		goto error;
@@ -306,7 +361,8 @@ error:
  * Add UST context to tracer.
  */
 int context_ust_add(struct ltt_ust_session *usess,
-		enum lttng_domain_type domain, struct lttng_event_context *ctx,
+		enum lttng_domain_type domain,
+		const struct lttng_event_context *ctx,
 		char *channel_name)
 {
 	int ret = LTTNG_OK;
@@ -339,7 +395,7 @@ int context_ust_add(struct ltt_ust_session *usess,
 		/* Add ctx all events, all channels */
 		cds_lfht_for_each_entry(chan_ht->ht, &iter.iter, uchan, node.node) {
 			ret = add_uctx_to_channel(usess, domain, uchan, ctx);
-			if (ret < 0) {
+			if (ret) {
 				ERR("Failed to add context to channel %s",
 						uchan->name);
 				continue;
@@ -349,10 +405,10 @@ int context_ust_add(struct ltt_ust_session *usess,
 	}
 
 	switch (ret) {
-	case -EEXIST:
-		ret = LTTNG_ERR_UST_CONTEXT_EXIST;
+	case LTTNG_ERR_UST_CONTEXT_EXIST:
 		break;
 	case -ENOMEM:
+	case -LTTNG_ERR_NOMEM:
 		ret = LTTNG_ERR_FATAL;
 		break;
 	case -EINVAL:
@@ -362,7 +418,11 @@ int context_ust_add(struct ltt_ust_session *usess,
 		ret = LTTNG_ERR_UNKNOWN_DOMAIN;
 		break;
 	default:
-		ret = LTTNG_OK;
+		if (ret != 0 && ret != LTTNG_OK) {
+			ret = ret > 0 ? ret : LTTNG_ERR_UNK;
+		} else {
+			ret = LTTNG_OK;
+		}
 		break;
 	}
 
