@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2011 - David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2011 David Goulet <dgoulet@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef _COMPAT_SOCKET_H
@@ -122,7 +112,7 @@ typedef struct ucred lttng_sock_cred;
 #define LTTNG_SOCK_GET_GID_CRED(c) LTTNG_REF(c)->gid
 #define LTTNG_SOCK_GET_PID_CRED(c) LTTNG_REF(c)->pid
 
-#elif (defined(__FreeBSD__) || defined(__CYGWIN__) || defined(__sun__))
+#elif (defined(__FreeBSD__) || defined(__CYGWIN__) || defined(__sun__) || defined(__APPLE__))
 
 struct lttng_sock_cred {
 	uid_t uid;
