@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2017 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ * Copyright (C) 2017 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
- * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License, version 2.1 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: LGPL-2.1-only
  *
- * This library is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
- * for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <lttng/notification/notification-internal.h>
@@ -161,9 +151,14 @@ struct lttng_notification_channel *lttng_notification_channel_create(
 	}
 
 	if (is_root || is_in_tracing_group) {
-		lttng_ctl_copy_string(sock_path,
+		ret = lttng_strncpy(sock_path,
 				DEFAULT_GLOBAL_NOTIFICATION_CHANNEL_UNIX_SOCK,
 				LTTNG_PATH_MAX);
+		if (ret) {
+			ret = -LTTNG_ERR_INVALID;
+			goto error;
+		}
+
 		ret = lttcomm_connect_unix_sock(sock_path);
 		if (ret >= 0) {
 			fd = ret;
