@@ -3,20 +3,10 @@
  *
  * LTTng filter generate intermediate representation
  *
- * Copyright 2012 - Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright 2012 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
- * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License, version 2.1 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: LGPL-2.1-only
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <stdio.h>
@@ -87,7 +77,7 @@ enum ir_load_string_type get_literal_string_type(const char *string)
 }
 
 static
-struct ir_op *make_op_load_string(char *string, enum ir_side side)
+struct ir_op *make_op_load_string(const char *string, enum ir_side side)
 {
 	struct ir_op *op;
 
@@ -197,7 +187,7 @@ struct ir_load_expression *create_load_expression(struct filter_node *node)
 {
 	struct ir_load_expression *load_exp;
 	struct ir_load_expression_op *load_exp_op, *prev_op;
-	char *str;
+	const char *str;
 
 	/* Get forward chain. */
 	node = load_expression_get_forward_chain(node);
