@@ -1,18 +1,7 @@
-# Copyright (C) - 2012 Christian Babeux <christian.babeux@efficios.com>
-# Copyright (C) - 2014 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+# Copyright (C) 2012 Christian Babeux <christian.babeux@efficios.com>
+# Copyright (C) 2014 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License, version 2 only, as
-# published by the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-# more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc., 51
-# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# SPDX-License-Identifier: GPL-2.0-only
 
 TESTDIR=${CURDIR}/../../..
 UST_EVENT_NAME="tp:tptest"
@@ -24,14 +13,6 @@ SLEEP_TIME=30
 
 source $TESTDIR/utils/utils.sh
 
-function lttng_create_session_uri
-{
-	# Create session with default path
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN create $SESSION_NAME \
-		-U net://localhost >/dev/null 2>&1
-	ok $? "Create session with default path"
-}
-
 function report_errors
 {
 	test_thread_error_string="$1"
@@ -89,7 +70,7 @@ function test_health
 	# Activate specific thread test
 	export ${test_thread_name}_${test_suffix}=1
 
-	# Spawn sessiond with preload healthexit lib
+	# Spawn sessiond with preloaded testpoint override lib
 	export LD_PRELOAD="$CURDIR/$SESSIOND_PRELOAD"
 
 	diag "Start session daemon"
