@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2012 - David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2012 David Goulet <dgoulet@efficios.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 2.1 of
- * the License.
+ * SPDX-License-Identifier: LGPL-2.1-only
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 #include <poll.h>
@@ -34,7 +23,7 @@ static void *th_event_minute(void *data)
 	/* Loop for 20 minutes */
 	for (i = 1; i < 21; i++) {
 		/* Sleep 60 seconds */
-		poll(NULL, 0, 60000);
+		(void) poll(NULL, 0, 60000);
 
 		/* 20 minutes tracepoint */
 		if ((i % 20) == 0) {
