@@ -1,6 +1,13 @@
 #!/bin/bash
+#
+# Copyright (C) 2016 Michael Jeanson <mjeanson@efficios.com>
+#
+# SPDX-License-Identifier: GPL-2.0-only
+#
 
-PGREP="$1"
+if [ -z $PGREP ]; then
+	PGREP=pgrep
+fi
 
 if [ x$LTTNG_TOOLS_TESTS_DISABLE_WARN_LTTNG_PROCESSES == x1 ]; then
 	exit
